@@ -327,9 +327,10 @@ check_swarp(void)
 {
     int ret = system(VISIR_SWARP_BIN " -v > /dev/null 2>/dev/null");
     if (ret != 0) {
-        cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
-                              "swarp not found in PATH");
-        return CPL_ERROR_UNSUPPORTED_MODE;
+        const char * path = getenv("PATH");
+        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
+                                     VISIR_SWARP_BIN " not found in PATH: %s",
+                                     path ? path : "<NULL>");
     }
     return CPL_ERROR_NONE;
 }
