@@ -1,4 +1,4 @@
-/* $Id: visir_util_clip.c,v 1.24 2012/02/02 10:27:28 jtaylor Exp $
+/* $Id: visir_util_clip.c,v 1.53 2013-05-23 14:33:29 jtaylor Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: jtaylor $
- * $Date: 2012/02/02 10:27:28 $
- * $Revision: 1.24 $
- * $Name: visir-3_5_1 $
+ * $Date: 2013-05-23 14:33:29 $
+ * $Revision: 1.53 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,11 @@
 #include "visir_recipe.h"
 
 #include <string.h>
+#include <strings.h> /* for strcasecmp */
+#include <math.h>
+#ifdef _OPENMP
+#include <omp.h>
+#endif
 
 /*-----------------------------------------------------------------------------
                                 Defines
@@ -47,7 +52,7 @@
 #define VISIR_UTIL_CLIP_KEEPFRAC 0.9
 #endif
 #ifndef VISIR_UTIL_CLIP_KAPPA
-#define VISIR_UTIL_CLIP_KAPPA 3.0
+#define VISIR_UTIL_CLIP_KAPPA 5.0
 #endif
 
 #ifndef VISIR_UTIL_CLIP_MAXITE
@@ -58,9 +63,38 @@
 #define VISIR_UTIL_CLIP_QC_MAP_MEAN   "ESO QC CONTRIBUTION MEAN"
 #define VISIR_UTIL_CLIP_QC_MAP_MEDIAN "ESO QC CONTRIBUTION MEDIAN"
 
+typedef enum {
+    VISIR_ERROR_SRC_IMG_STDEV,
+    VISIR_ERROR_SRC_TIMESERIES,
+} visir_error_source;
+
+/* used by sigmaclipping to cache some expensive values */
+static int prnok_nz = 0;
+static double * prnok = NULL;
+
 /*-----------------------------------------------------------------------------
                             Private Functions prototypes
  -----------------------------------------------------------------------------*/
+static double median_abs_dev(const cpl_image * img)
+{
+    /* use cpl_image_get_mad when cpl 6.2 only supported version */
+    const size_t npix = cpl_image_get_size_x(img) * cpl_image_get_size_y(img);
+    const double median = cpl_image_get_median(img);
+    const cpl_mask * bpm = cpl_image_get_bpm_const(img);
+    const cpl_binary * bpm_ = cpl_mask_get_data_const(bpm);
+    const float * data = cpl_image_get_data_float_const(img);
+    cpl_array * tmp = cpl_array_new(npix, CPL_TYPE_FLOAT);
+
+    for (size_t i = 0; i < npix; i++)
+        if (bpm_[i] != CPL_BINARY_0)
+            cpl_array_set_invalid(tmp, i);
+        else
+            cpl_array_set_float(tmp, i, fabs(data[i] - median));
+
+    double mad = cpl_array_get_median(tmp);
+    cpl_array_delete(tmp);
+    return mad;
+}
 
 static
 cpl_error_code visir_util_clip_kappa_sigma_double(cpl_imagelist *,
@@ -81,6 +115,7 @@ cpl_error_code visir_util_clip_kappa_sigma_int(cpl_imagelist *,
 static cpl_error_code visir_util_clip_one(cpl_frameset *,
                                           irplib_framelist *,
                                           irplib_framelist *,
+                                          const cpl_mask *,
                                           int, cpl_boolean,
                                           const cpl_parameterlist *);
 
@@ -90,6 +125,9 @@ static cpl_error_code visir_util_clip_kappa_sigma(cpl_imagelist *,
                                                   const int * shifts);
 
 
+#ifdef VISIR_CHAIN
+#define cpl_plugin_get_info visir_util_clip_get_info
+#endif
 cpl_recipe_define(visir_util_clip, VISIR_BINARY_VERSION,                
                   "Lars Lundin", PACKAGE_BUGREPORT, "2011",
                   "Kappa-sigma clipping of outliers for each pixel",
@@ -156,6 +194,7 @@ cpl_error_code visir_util_clip_fill_parameterlist(cpl_parameterlist * self)
                                        "maxite", VISIR_UTIL_CLIP_MAXITE, NULL,
                                        context, "Max number of kappa-sigma "
                                        "clipping iterations");
+    cpl_ensure_code(!err, err);
 
     /* --shift-beams */
     err = irplib_parameterlist_set_bool(self, PACKAGE, RECIPE_STRING,
@@ -164,11 +203,24 @@ cpl_error_code visir_util_clip_fill_parameterlist(cpl_parameterlist * self)
                                         "object defined in CRPIX[12]");
     cpl_ensure_code(!err, err);
 
-    /* --error */
-    err = irplib_parameterlist_set_bool(self, PACKAGE, RECIPE_STRING,
-                                        "error", CPL_TRUE, NULL,
-                                        context, "Output clipped standard "
-                                        "deviation as error map");
+    /* FIXME: add an adaptive mode that just requires n pixels
+     * and prefers timeseries mode */
+    /* --error-source */
+    err = irplib_parameterlist_set_string(self, PACKAGE, RECIPE_STRING,
+                                          "error-source", "img-stdev", NULL,
+                                          context, "Defines the way errors "
+                                          "are generated:\n"
+                                          " img-stdev: stdev of image\n"
+                                          " timeseries: stdev of each pixel "
+                                          "over the time axis of the cube\n"
+                                          " none: no error generation");
+    cpl_ensure_code(!err, err);
+
+    /* --error-out-type */
+    err = irplib_parameterlist_set_string(self, PACKAGE, RECIPE_STRING,
+                                          "error-out-type", "error", NULL,
+                                          context, "Output clipped error as "
+                                          "error, variance, weight or none");
     cpl_ensure_code(!err, err);
 
     /* --badimage */
@@ -199,31 +251,59 @@ static int visir_util_clip(cpl_frameset            * framelist,
     irplib_framelist * allframes = NULL;
     irplib_framelist * rawframes = NULL;
     irplib_framelist * bpmframes = NULL;
-    cpl_size           i, n, nbad = 0;
-    
+    cpl_size           n, nbad = 0;
+    cpl_mask * static_mask = NULL;
+    prnok = NULL;
+
+#ifdef _OPENMP
+    omp_set_num_threads(visir_get_num_threads(CPL_FALSE));
+#endif
 
     /* Identify the RAW and CALIB frames in the input frameset */
     skip_if (visir_dfs_set_groups(framelist));
+    cpl_fits_set_mode(CPL_FITS_START_CACHING);
 
     /* Objects observation */
     allframes = irplib_framelist_cast(framelist);
     skip_if(allframes == NULL);
-    rawframes = irplib_framelist_extract(allframes, VISIR_UTIL_INPUTS_RAW);
+    rawframes =
+        irplib_framelist_extract_regexp(allframes, "^("VISIR_UTIL_INPUTS_RAW
+                                        "|"VISIR_UTIL_CORRECTED")$",
+                                        CPL_FALSE);
     skip_if (rawframes == NULL);
-    bpmframes = irplib_framelist_extract(allframes, VISIR_CALIB_BPM);
+    bpmframes = irplib_framelist_extract_regexp(allframes, "^(" VISIR_CALIB_BPM
+                                                ")$", CPL_FALSE);
+
     if (bpmframes == NULL)
         cpl_errorstate_set(cleanstate);
     else
         nbad = irplib_framelist_get_size(bpmframes);
+
+    {
+        cpl_frame * static_frm = cpl_frameset_find(framelist,
+                                                   VISIR_CALIB_STATIC_MASK);
+        if (static_frm) {
+            const char * fn = cpl_frame_get_filename(static_frm);
+            static_mask = cpl_mask_load(fn, 0, 0);
+            if (!static_mask) {
+                cpl_errorstate_set(cleanstate);
+                static_mask = cpl_mask_load(fn, 0, 1);
+                if (!static_mask)
+                    cpl_errorstate_set(cleanstate);
+            }
+        }
+    }
     
     n = irplib_framelist_get_size(rawframes);
     error_if(nbad != n && nbad != 1 && nbad != 0, CPL_ERROR_INCOMPATIBLE_INPUT,
-             "%d raw-frames <=> %d bpm frames", n, nbad);
+             "%d raw-frames <=> %d bpm frames", (int)n, (int)nbad);
 
-#ifdef _OPENMP
-#pragma omp parallel for private(i)
+    /* cpl 6.0 fits caching is buggy when paralizing over many files */
+#if defined _OPENMP && \
+    defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(6, 1, 0)
+#pragma omp parallel for
 #endif
-    for (i = 0; i < n; i++) {
+    for (cpl_size i = 0; i < n; i++) {
         if (!didfail) {
 
             /* The total number of iterations must be pre-determined for the
@@ -233,7 +313,8 @@ static int visir_util_clip(cpl_frameset            * framelist,
                iteration can cause an error to be dumped, but it is not
                worse than checking on a thread-local state, e.g. errori. */
 
-            if (visir_util_clip_one(framelist, rawframes, bpmframes, i,
+            if (visir_util_clip_one(framelist, rawframes, bpmframes,
+                                    static_mask, i,
                                     nbad == 1, parlist)) {
                 const cpl_error_code errori = cpl_error_set_where(cpl_func);
 #ifdef _OPENMP
@@ -248,22 +329,19 @@ static int visir_util_clip(cpl_frameset            * framelist,
         }
     }
 
-    error_if(didfail, didfail, "Failed to clip %d frame(s)", n);
+    error_if(didfail, didfail, "Failed to clip %d frame(s)", (int)n);
 
     end_skip;
 
     irplib_framelist_delete(allframes);
     irplib_framelist_delete(rawframes);
     irplib_framelist_delete(bpmframes);
+    cpl_free(prnok);
+    prnok = NULL;
 
     return cpl_error_get_code();
 }
 
-static inline int round_to_int(const double x)
-{
-    return x >= 0 ? (int)(x + 0.5) : (int)(x - 0.5);
-}
-
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -271,7 +349,8 @@ static inline int round_to_int(const double x)
   @param    framelist   The frameset to append products to
   @param    rawframes   The raw frames list (will load the propertylist)
   @param    bpmframes   The bpm frames list (will load the propertylist)
-  @param    i           The frame to process (0 for first)
+  @param    static_mask Static bad pixel map or NULL
+  @param    iframe      The frame to process (0 for first)
   @param    bshared     CPL_TRUE iff the bpm is shared among all raw frames
   @param    parlist     The parameters list
   @return   CPL_ERROR_NONE iff OK, otherwise the relevant CPL error code.
@@ -280,34 +359,35 @@ static inline int round_to_int(const double x)
 static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
                                           irplib_framelist * rawframes,
                                           irplib_framelist * bpmframes,
-                                          int i, cpl_boolean bshared,
+                                          const cpl_mask * static_mask,
+                                          int iframe, cpl_boolean bshared,
                                           const cpl_parameterlist * parlist)
 {
 
     cpl_frameset  * products   = cpl_frameset_new();
     cpl_frameset  * usedframes = cpl_frameset_new();
-    const cpl_frame * frame;
 
-    char * bpmname = cpl_sprintf(RECIPE_STRING "_bpm_%03d" CPL_DFS_FITS, i);
-    char * mapname = cpl_sprintf(RECIPE_STRING "_map_%03d" CPL_DFS_FITS, i);
-    char * errname = cpl_sprintf(RECIPE_STRING "_error_%03d" CPL_DFS_FITS, i);
+    char * bpmname = cpl_sprintf(RECIPE_STRING "_bpm_%03d" CPL_DFS_FITS, iframe);
+    char * mapname = cpl_sprintf(RECIPE_STRING "_map_%03d" CPL_DFS_FITS, iframe);
+    char * errname = cpl_sprintf(RECIPE_STRING "_error_%03d" CPL_DFS_FITS, iframe);
 
     const int            n = irplib_framelist_get_size(rawframes);
     cpl_frameset       * rawone = cpl_frameset_new();
     cpl_frameset       * bpmone = cpl_frameset_new();
     cpl_frame * rawframe =
-        cpl_frame_duplicate(irplib_framelist_get_const(rawframes, i));
+        cpl_frame_duplicate(irplib_framelist_get_const(rawframes, iframe));
     cpl_frame * bpmframe = bpmframes ?
         cpl_frame_duplicate(irplib_framelist_get_const(bpmframes,
-                                                       bshared ? 0 : i))
+                                                       bshared ? 0 : iframe))
         : NULL;
     const cpl_error_code errr = cpl_frameset_insert(rawone, rawframe);
     const cpl_error_code errb = bpmframe ?
         cpl_frameset_insert(bpmone, bpmframe) : CPL_ERROR_NONE;
 
-    const cpl_boolean berr = irplib_parameterlist_get_bool(parlist, PACKAGE,
-                                                           RECIPE_STRING,
-                                                           "error");
+    const char * serr = irplib_parameterlist_get_string(parlist, PACKAGE,
+                                                        RECIPE_STRING,
+                                                        "error-out-type");
+    cpl_boolean berr = serr && strcasecmp(serr, "none") != 0;
 
     const cpl_boolean bshifts = irplib_parameterlist_get_bool(parlist, PACKAGE,
                                                               RECIPE_STRING,
@@ -317,45 +397,60 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
                                                             RECIPE_STRING,
                                                             "badimage");
 
+    const char * serr_src =
+        irplib_parameterlist_get_string(parlist, PACKAGE, RECIPE_STRING,
+                                        "error-source");
+
     cpl_imagelist * devlist = cpl_imagelist_new();
     cpl_imagelist * rawlist =
         cpl_imagelist_load_frameset(rawone, CPL_TYPE_UNSPECIFIED, 0, -1);
     cpl_imagelist * bpmlist = bpmframe ?
         cpl_imagelist_load_frameset(bpmone, CPL_TYPE_INT, 0, -1) : NULL;
 
-#ifdef CPL_BPP_IEEE_FLOAT
-/* FIXME: Use just CPL_TYPE_UNSPECIFIED once CPL 5.3.1 is no longer supported */
-    const cpl_type_bpp stdev_save_type = CPL_TYPE_UNSPECIFIED;
-#else
-    const cpl_type_bpp stdev_save_type = cpl_image_get_type
-        (cpl_imagelist_get_const(rawlist, 0)) == CPL_TYPE_DOUBLE
-        ? CPL_BPP_IEEE_DOUBLE : CPL_BPP_IEEE_FLOAT;
-#endif
-
     cpl_mask      * bpm    = NULL;
-    cpl_image     * newbpmimg = NULL;
     cpl_image     * map = NULL;
     cpl_propertylist * qclist = cpl_propertylist_new();
     cpl_propertylist * xtlist = cpl_propertylist_new();
 
     const int       m = rawlist ? cpl_imagelist_get_size(rawlist) : 0;
     const int       nbpm = bpmlist ? cpl_imagelist_get_size(bpmlist) : 0;
-    int             j;
     int             mapmax, mapmean, mapmedian;
     int *           shifts = cpl_calloc(m * 2, sizeof(int));
+    double fx = 0, fy = 0; /* avoid false uninit warning */
+
+    const char * err_procatg = "ERROR_MAP";
+    visir_error_type terr = VISIR_ERROR;
+    visir_error_source err_src = VISIR_ERROR_SRC_IMG_STDEV;
+    if (berr && strcmp(serr, "variance") == 0) {
+        terr = VISIR_VARIANCE;
+        err_procatg = "VARIANCE_MAP";
+    }
+    else if (berr && strcmp(serr, "weight") == 0) {
+        terr = VISIR_WEIGHT;
+        err_procatg = "WEIGHT_MAP";
+    }
+
+    if (serr_src && !strcmp(serr_src, "img-stdev"))
+        err_src = VISIR_ERROR_SRC_IMG_STDEV;
+    else if (serr_src && !strcmp(serr_src, "timeseries"))
+        err_src = VISIR_ERROR_SRC_TIMESERIES;
+    else if (serr_src && !strcasecmp(serr_src, "none"))
+        berr = CPL_FALSE;
+    else if (serr)
+        error_if(1, CPL_ERROR_ILLEGAL_INPUT, "Unknown error-source: %s",
+                 serr_src);
 
     skip_if(0);
 
     for (int e = 0; bshifts && e < m; e++) {
         const cpl_propertylist * plist;
         double crpix1, crpix2;
-        double fx, fy;
         cpl_errorstate prestate = cpl_errorstate_get();
 
-        irplib_framelist_load_propertylist(rawframes, i, e + 1, "^("
+        irplib_framelist_load_propertylist(rawframes, iframe, e + 1, "^("
                                            IRPLIB_PFITS_WCS_REGEXP ")$",
                                            CPL_FALSE);
-        plist = irplib_framelist_get_propertylist_const(rawframes, i);
+        plist = irplib_framelist_get_propertylist_const(rawframes, iframe);
         if (!cpl_propertylist_has(plist, "CRPIX1") ||
             !cpl_propertylist_has(plist, "CRPIX2")) {
             cpl_errorstate_set(prestate);
@@ -371,8 +466,8 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
             fx = crpix1;
             fy = crpix2;
         } else {
-            shifts[e * 2]     = round_to_int(-(fx - crpix1));
-            shifts[e * 2 + 1] = round_to_int(-(fy - crpix2));
+            shifts[e * 2]     = visir_round_to_int(-(fx - crpix1));
+            shifts[e * 2 + 1] = visir_round_to_int(-(fy - crpix2));
         }
         cpl_msg_debug(cpl_func, "CRPIX shifts %d %d, %f %f", shifts[e * 2],
                       shifts[e * 2 + 1], crpix1 - fx, crpix2 - fy);
@@ -384,14 +479,14 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
     bug_if(errb);
 
     error_if(nbpm != 1 && nbpm != m && nbpm != 0, CPL_ERROR_INCOMPATIBLE_INPUT,
-             "Frame-pair %d/%d: %d image(s) <=> %d bad pixel map(s)", 1+i, n,
-             m, (int)cpl_imagelist_get_size(bpmlist));
+             "Frame-pair %d/%d: %d image(s) <=> %d bad pixel map(s)",
+             1+iframe, n, m, (int)cpl_imagelist_get_size(bpmlist));
 
     bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(rawframe)));
     if (bpmframe)
         bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(bpmframe)));
 
-    for (j = 0; bpmframe && j < m; j++) {
+    for (int j = 0; bpmframe && j < m; j++) {
         const cpl_image * bpmimg =
             cpl_imagelist_get_const(bpmlist, nbpm > 1 ? j : 0);
         cpl_image * rawimg       = cpl_imagelist_get(rawlist, j);
@@ -399,6 +494,10 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
         cpl_mask_delete(bpm);
         bpm = cpl_mask_threshold_image_create(bpmimg, 0.5, FLT_MAX);
 
+        /* merge static mask */
+        if (static_mask)
+            cpl_mask_or(bpm, static_mask);
+
         bug_if(cpl_image_reject_from_mask(rawimg, bpm));
 
     }
@@ -411,14 +510,11 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
         cpl_image * img = cpl_imagelist_get(rawlist, 0);
         const int total_size =
             cpl_image_get_size_x(img) * cpl_image_get_size_y(img);
-        cpl_binary * _allbad = cpl_malloc(sizeof(cpl_binary) * total_size);
-        cpl_mask * allbad;
+        cpl_mask * allbad = cpl_mask_new(cpl_image_get_size_x(img),
+                                         cpl_image_get_size_y(img));
+        cpl_mask_not(allbad);
 
-        memset(_allbad, CPL_BINARY_1, sizeof(cpl_binary) * total_size);
-        allbad = cpl_mask_wrap(cpl_image_get_size_x(img),
-                               cpl_image_get_size_y(img), _allbad);
-
-        for (int i = 0; i < cpl_imagelist_get_size(rawlist); i++) {
+        for (cpl_size i = 0; i < cpl_imagelist_get_size(rawlist); i++) {
             cpl_mask * mask = cpl_image_get_bpm(cpl_imagelist_get(rawlist, i));
             img = cpl_imagelist_get(rawlist, i);
             if (badimage < (double)cpl_mask_count(mask) / total_size)
@@ -448,41 +544,82 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
     bug_if(cpl_propertylist_set_comment(qclist, VISIR_UTIL_CLIP_QC_MAP_MEDIAN,
                                         "The median contribution on a pixel"));
 
-#ifdef VISIR_MIME_CUBE_SUPPORT
-    /* Pending closure of DFS 10467 */
-#error "Save data as a cube in the primary data unit"
-#else
-    /* Do not save data in the primary data unit (DFS10475) */
-    if (bpmframe)
-        skip_if(irplib_dfs_save_propertylist(products, parlist, usedframes,
-                                             RECIPE_STRING,
-                                             VISIR_IMG_CLIPPED_PROCATG, qclist,
-                                             NULL, visir_pipe_id, bpmname));
+    skip_if(irplib_dfs_save_propertylist(products, parlist, usedframes,
+                                         RECIPE_STRING,
+                                         VISIR_IMG_CLIPPED_PROCATG, qclist,
+                                         NULL, visir_pipe_id, bpmname));
 
     if (berr)
         skip_if(irplib_dfs_save_propertylist(products, parlist, usedframes,
                                              RECIPE_STRING,
-                                             "ERROR_MAP", qclist,
+                                             err_procatg, qclist,
                                              NULL, visir_pipe_id, errname));
 
-    for (j = 0; j < m; j++) {
+    for (int j = 0; j < m; j++) {
         cpl_image * rawimg       = cpl_imagelist_get(rawlist, j);
+        const cpl_mask * newbpm  = cpl_image_get_bpm_const(rawimg);
+        const cpl_size npix =
+            cpl_image_get_size_x(rawimg) * cpl_image_get_size_y(rawimg);
+        cpl_image * err;
 
-        if (bpmframe) {
-            const cpl_mask * newbpm  = cpl_image_get_bpm_const(rawimg);
+        skip_if(cpl_mask_save(newbpm, bpmname, NULL, CPL_IO_EXTEND));
 
-            cpl_image_delete(newbpmimg);
-            newbpmimg = cpl_image_new_from_mask(newbpm);
+        if (berr == CPL_FALSE)
+            continue;
 
-            skip_if(cpl_image_save(newbpmimg, bpmname, CPL_BPP_8_UNSIGNED,
-                                   NULL, CPL_IO_EXTEND));
+        if (err_src == VISIR_ERROR_SRC_IMG_STDEV) {
+            double bkgsigma;
+            err = cpl_image_new(cpl_image_get_size_x(rawimg),
+                                cpl_image_get_size_y(rawimg),
+                                CPL_TYPE_FLOAT);
+            if (cpl_mask_count(cpl_image_get_bpm_const(rawimg)) == npix)
+                bkgsigma = INFINITY;
+            else {
+                cpl_image * dupl = cpl_image_cast(rawimg, CPL_TYPE_FLOAT);
+                const double median = cpl_image_get_median(dupl);
+                const double sigma = cpl_image_get_stdev(dupl);
+                const double kappa =
+                    irplib_parameterlist_get_double(parlist, PACKAGE,
+                                                    RECIPE_STRING, "kappa");
+                /* mask out signal */
+                cpl_mask * mask =
+                    cpl_mask_threshold_image_create(dupl, median - kappa * sigma,
+                                                    median + kappa * sigma);
+                skip_if(0);
+                /* for zero image testcase ... */
+                if (!cpl_mask_is_empty(mask))
+                    cpl_mask_not(mask);
+                cpl_mask_or(mask, cpl_image_get_bpm(dupl));
+                cpl_image_reject_from_mask(dupl, mask);
+                /* conversion mad -> stdev for gaussian noise */
+                bkgsigma = median_abs_dev(dupl) * 1.4826;
+                cpl_msg_debug(cpl_func, "%d: sigma %.4f", j, bkgsigma);
+                cpl_image_delete(dupl);
+                cpl_mask_delete(mask);
+                skip_if(0);
+            }
+            cpl_image_add_scalar(err, bkgsigma);
         }
+        else if (err_src == VISIR_ERROR_SRC_TIMESERIES)
+            err = cpl_image_duplicate(cpl_imagelist_get_const(devlist, 1));
+        else
+            bug_if(1);
 
-        if (berr)
-            skip_if(cpl_image_save(cpl_imagelist_get_const(devlist, 1), errname,
-                                   stdev_save_type, NULL, CPL_IO_EXTEND));
+        cpl_image_reject_from_mask(err, newbpm);
+        if (terr == VISIR_ERROR)
+            cpl_image_fill_rejected(err, INFINITY);
+        else if (terr == VISIR_VARIANCE) {
+            cpl_image_power(err, 2);
+            cpl_image_fill_rejected(err, INFINITY);
+        }
+        else if (terr == VISIR_WEIGHT) {
+            cpl_image_power(err, -2);
+            cpl_image_fill_rejected(err, 0);
+        }
+        cpl_image_save(err, errname, CPL_TYPE_FLOAT, NULL, CPL_IO_EXTEND);
+        cpl_image_delete(err);
+        skip_if(0);
     }
-#endif
 
     skip_if(irplib_dfs_save_image(products, parlist, usedframes,
                                   map, m < 256 ? CPL_BPP_8_UNSIGNED
@@ -495,30 +632,19 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
                                           "DEVIATION MAP"));
 
     skip_if(cpl_image_save(cpl_imagelist_get_const(devlist, 0), mapname,
-                           stdev_save_type, xtlist, CPL_IO_EXTEND));
+                           CPL_TYPE_FLOAT, xtlist, CPL_IO_EXTEND));
 
     bug_if(cpl_propertylist_update_string(xtlist, "EXTNAME", "CLIPPED STANDARD "
                                           "DEVIATION MAP"));
 
     skip_if(cpl_image_save(cpl_imagelist_get_const(devlist, 1), mapname,
-                           stdev_save_type, xtlist, CPL_IO_EXTEND));
-
-    for (frame = cpl_frameset_get_first_const(products);
-         frame != NULL;
-         frame = cpl_frameset_get_next_const(products)) {
-        cpl_frame * copy = cpl_frame_duplicate(frame);
-        cpl_error_code error;
+                           CPL_TYPE_FLOAT, xtlist, CPL_IO_EXTEND));
 
 #ifdef _OPENMP
 #pragma omp critical(visir_util_clip_one)
 #endif
-        error = cpl_frameset_insert(framelist, copy);
-
-        if (error) break;
-    }
-
-    bug_if(frame != NULL);
-
+    FOR_EACH_FRAMESET_C(frame, products)
+        cpl_frameset_insert(framelist, cpl_frame_duplicate(frame));
 
     end_skip;
 
@@ -536,7 +662,6 @@ static cpl_error_code visir_util_clip_one(cpl_frameset * framelist,
     cpl_propertylist_delete(xtlist);
 
     cpl_mask_delete(bpm);
-    cpl_image_delete(newbpmimg);
     cpl_image_delete(map);
     cpl_imagelist_delete(rawlist);
     cpl_imagelist_delete(bpmlist);
