@@ -1,5 +1,7 @@
 from __future__ import with_statement
 # must succeed, min requirement py 2.5
+from __future__ import absolute_import
+from __future__ import print_function
 import sys
 from collections import defaultdict
 from visir_utils import log_invocation, Future
@@ -30,9 +32,9 @@ try:
     import wx
 
     import_success = True
-except (ImportError, NameError), e:
+except (ImportError, NameError) as e:
     import_success = False
-    print "Error importing modules: ", str(e)
+    print("Error importing modules: ", str(e))
 
 
 def myspanignore(self, event):
@@ -236,7 +238,7 @@ class DataPlotterManager:
             sign = s[0]
             bp += '%s:%d,%d,%d;' % (sign, x, y, w)
 
-        print bp, ret, wx.ID_OK
+        print(bp, ret, wx.ID_OK)
         if bp and ret == wx.ID_OK:
             self.parWidgets['beampos'].set_value(bp)
         dlg.Destroy()
@@ -555,7 +557,7 @@ if __name__ == '__main__':
         else:
             app.passProductsThrough()
     except (ImportError, NameError) as e:
-        print 'Error:', e
+        print('Error:', e)
         app.passProductsThrough()
 
     app.print_outputs()
