@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import sys
 import wx
 from wx import xrc
@@ -172,7 +173,7 @@ A checked box represents a positive beam and an unchecked box a negative beam.
         if not add and ind >= 0:
             self.beamBox.DeleteItem(ind)
         else:
-            idx = self.beamBox.InsertStringItem(sys.maxint, '')
+            idx = self.beamBox.InsertStringItem(sys.maxsize, '')
             sign = self.check_sign(*data, d=self.data)
             if sign == 'pos':
                 self.beamBox.CheckItem(idx)
