@@ -805,7 +805,7 @@ int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
         /* adapt VLT-TRE-VIS-14321-5046 measurements to new detector size and
          * pfov */
         self->factor *= 0.127 / 0.0757;
-        self->ld /= 0.127 / 0.0757;
+        self->ld *= 0.127 / 0.0757;
         /* drs: b0 at pixel 1, bm at 128.5, b1 at 256 with pfov 0.127
          * aqu: bm at 512.5, b1 at 1024 with pfov 0.757
          * => d = 128.5 / tan(b1 - bm) -> (b1 - bm)_new = atan((512.5 / 1.6777) / d) */
